@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-#    Copyright (C) 2013-2016 Christian T. Jacobs.
+#    Copyright (C) 2013-2017 Christian Thomas Jacobs.
 
 #    This file is part of PyQSO.
 
@@ -19,24 +19,24 @@
 
 from gi.repository import Gtk
 import logging
+import sqlite3 as sqlite
 
 
-class Awards(Gtk.VBox):
+class Awards:
 
     """ A tool for tracking progress towards an award. Currently this only supports the DXCC award.
     For more information visit http://www.arrl.org/dxcc """
 
-    def __init__(self, parent):
+    def __init__(self, application):
         """ Set up a table for progress tracking purposes.
 
-        :arg parent: The parent Gtk window.
+        :arg application: The PyQSO application containing the main Gtk window, etc.
         """
-        # TODO: This only considers the DXCC award for now.
-        logging.debug("New Awards instance created!")
+        # TODO: Add more awards. This only considers the DXCC award for now.
+        logging.debug("Setting up awards table...")
 
-        Gtk.VBox.__init__(self, spacing=2)
-
-        self.parent = parent
+        self.application = application
+        self.builder = self.application.builder
 
         self.bands = ["70cm", "2m", "6m", "10m", "12m", "15m", "17m", "20m", "30m", "40m", "80m", "160m"]
         self.modes = ["Phone", "CW", "Digital", "Mixed"]
@@ -44,9 +44,9 @@ class Awards(Gtk.VBox):
         data_types = [str] + [int]*len(self.bands)
         self.awards = Gtk.ListStore(*data_types)
 
-        # The main table for the awards
-        self.treeview = Gtk.TreeView(self.awards)
-        # A separate, empty column just for the mode names
+        # The main table for the awards.
+        self.treeview = Gtk.TreeView(model=self.awards)
+        # A separate, empty column just for the mode names.
         renderer = Gtk.CellRendererText()
         column = Gtk.TreeViewColumn("Modes", renderer, text=0)
         column.set_clickable(False)
@@ -60,34 +60,37 @@ class Awards(Gtk.VBox):
             column.set_clickable(False)
             self.treeview.append_column(column)
 
-        # Add a label to inform the user that this only considers the DXCC award for now.
-        label = Gtk.Label(halign=Gtk.Align.START)
-        label.set_markup("<span size=\"x-large\">%s</span>" % "DXCC Award")
-        self.pack_start(label, False, False, 4)
-        # Show the table in the Awards tab
-        self.add(self.treeview)
-        self.show_all()
+        # Show the table in the Awards tab.
+        self.builder.get_object("awards").add(self.treeview)
+        self.builder.get_object("awards").show_all()
 
         logging.debug("Awards table set up successfully.")
 
-        self.count()
+        self.count(self.application.logbook)
 
         return
 
-    def count(self):
-        """ Update the table for progress tracking. """
+    def count(self, logbook):
+        """ Update the table for progress tracking.
+
+        :arg logbook: The logbook containing logs which in turn contain QSOs.
+        :returns: A list of lists containing the QSO counts for different modes and bands.
+        :rtype: list
+        """
 
         logging.debug("Counting the band/mode combinations for the awards table...")
-        # Wipe everything and start again
+
+        # Wipe everything and start again.
         self.awards.clear()
+
         # For each mode, add a new list for holding the totals, and initialise the values to zero.
         count = []
         for i in range(0, len(self.bands)):
             count.append([0]*len(self.bands))
 
-        for log in self.parent.logbook.logs:
-            records = log.get_all_records()
-            if(records is not None):
+        for log in logbook.logs:
+            try:
+                records = log.records
                 for r in records:
                     if(r["BAND"] is not None and r["MODE"] is not None):
                         if(r["BAND"].lower() in self.bands and r["MODE"] != ""):
@@ -100,11 +103,15 @@ class Awards(Gtk.VBox):
                             else:
                                 # FIXME: This assumes that all the other modes in the ADIF list are digital modes. Is this the case?
                                 count[2][band] += 1
-                            count[3][band] += 1  # Keep the total of each column in the "Mixed" mode
-            else:
+                            count[3][band] += 1  # Keep the total of each column in the "Mixed" mode.
+
+            except sqlite.Error as e:
                 logging.error("Could not update the awards table for '%s' because of a database error." % log.name)
-        # Insert the rows containing the totals
+                logging.exception(e)
+
+        # Insert the rows containing the totals.
         for i in range(0, len(self.modes)):
             self.awards.append([self.modes[i]] + count[i])
+
         logging.debug("Awards table updated.")
-        return
+        return count
