@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-#    Copyright (C) 2013-2016 Christian T. Jacobs.
+#    Copyright (C) 2013-2017 Christian Thomas Jacobs.
 
 #    This file is part of PyQSO.
 
@@ -17,136 +17,121 @@
 #    You should have received a copy of the GNU General Public License
 #    along with PyQSO.  If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gtk, GObject
+from gi.repository import Gtk, GObject, Gdk
 import logging
 import telnetlib
-import unittest
-import unittest.mock
-import configparser
+try:
+    import configparser
+except ImportError:
+    import ConfigParser as configparser
 import os.path
 
-from pyqso.telnet_connection_dialog import *
+from pyqso.telnet_connection_dialog import TelnetConnectionDialog
+from pyqso.auxiliary_dialogs import error
 
 BOOKMARKS_FILE = os.path.expanduser('~/.config/pyqso/bookmarks.ini')
 
 
-class DXCluster(Gtk.VBox):
+class DXCluster:
 
     """ A tool for connecting to a DX cluster (specifically Telnet-based DX clusters). """
 
-    def __init__(self, parent):
-        """ Set up the DX cluster's Gtk.VBox, and set up a timer so that PyQSO can retrieve new data from the Telnet server every few seconds.
+    def __init__(self, application):
+        """ Set up the DX cluster, and set up a timer so that PyQSO can retrieve new data from the Telnet server every few seconds.
 
-        :arg parent: The parent Gtk window.
+        :arg application: The PyQSO application containing the main Gtk window, etc.
         """
+
         logging.debug("Setting up the DX cluster...")
-        Gtk.VBox.__init__(self, spacing=2)
 
+        self.application = application
+        self.builder = self.application.builder
         self.connection = None
-        self.parent = parent
 
-        # Set up the menubar
-        self.menubar = Gtk.MenuBar()
+        # Connect signals.
+        self.builder.get_object("mitem_new").connect("activate", self.new_server)
+        self.builder.get_object("mitem_disconnect").connect("activate", self.telnet_disconnect)
+        self.builder.get_object("send").connect("clicked", self.telnet_send_command)
+        self.builder.get_object("command").connect("key-press-event", self.on_command_key_press)
 
-        self.items = {}
-
-        # CONNECTION ######
-        mitem_connection = Gtk.MenuItem(label="Connection")
-        self.menubar.append(mitem_connection)
-        subm_connection = Gtk.Menu()
-        mitem_connection.set_submenu(subm_connection)
-
-        # Connect
-        mitem_connect = Gtk.ImageMenuItem(label="Connect to Telnet Server")
-        icon = Gtk.Image()
-        icon.set_from_stock(Gtk.STOCK_CONNECT, Gtk.IconSize.MENU)
-        mitem_connect.set_image(icon)
-        subm_connection.append(mitem_connect)
-        self.items["CONNECT"] = mitem_connect
-
-        subm_connect = Gtk.Menu()
-
-        # New
-        mitem_new = Gtk.MenuItem(label="New...")
-        mitem_new.connect("activate", self.new_server)
-        subm_connect.append(mitem_new)
-
-        # From Bookmark
-        mitem_bookmark = Gtk.MenuItem(label="From Bookmark")
-        self.subm_bookmarks = Gtk.Menu()
-        mitem_bookmark.set_submenu(self.subm_bookmarks)
-        self._populate_bookmarks()
-        subm_connect.append(mitem_bookmark)
-
-        mitem_connect.set_submenu(subm_connect)
-
-        # Disconnect
-        mitem_disconnect = Gtk.ImageMenuItem(label="Disconnect from Telnet Server")
-        icon = Gtk.Image()
-        icon.set_from_stock(Gtk.STOCK_DISCONNECT, Gtk.IconSize.MENU)
-        mitem_disconnect.set_image(icon)
-        mitem_disconnect.connect("activate", self.telnet_disconnect)
-        subm_connection.append(mitem_disconnect)
-        self.items["DISCONNECT"] = mitem_disconnect
-
-        self.pack_start(self.menubar, False, False, 0)
-
-        # A TextView object to display the output from the Telnet server.
-        self.renderer = Gtk.TextView()
-        self.renderer.set_editable(False)
-        self.renderer.set_cursor_visible(False)
-        sw = Gtk.ScrolledWindow()
-        sw.set_shadow_type(Gtk.ShadowType.ETCHED_IN)
-        sw.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
-        sw.add(self.renderer)
+        # Get the text renderer and its buffer.
+        self.renderer = self.builder.get_object("renderer")
         self.buffer = self.renderer.get_buffer()
-        self.pack_start(sw, True, True, 0)
-
-        # Set up the command box.
-        self.commandbox = Gtk.HBox(spacing=2)
-        self.command = Gtk.Entry()
-        self.commandbox.pack_start(self.command, True, True, 0)
-        self.send = Gtk.Button(label="Send Command")
-        self.send.connect("clicked", self.telnet_send_command)
-        self.commandbox.pack_start(self.send, False, False, 0)
-        self.pack_start(self.commandbox, False, False, 0)
 
+        # Items whose sensitivity may change.
+        self.items = {}
+        self.items["CONNECT"] = self.builder.get_object("mitem_connect")
+        self.items["DISCONNECT"] = self.builder.get_object("mitem_disconnect")
+        self.items["SEND"] = self.builder.get_object("send")
         self.set_items_sensitive(True)
 
-        self.show_all()
+        self.populate_bookmarks()
 
         logging.debug("DX cluster ready!")
 
         return
 
-    def new_server(self, widget=None):
-        """ Get Telnet server host and login details specified in the Gtk.Entry boxes in the TelnetConnectionDialog and attempt a connection. """
-        dialog = TelnetConnectionDialog(self.parent)
-        response = dialog.run()
+    def on_command_key_press(self, widget, event, data=None):
+        """ If the Return key is pressed when the focus is on the command box, then send whatever command the user has entered. """
+        if(event.keyval == Gdk.KEY_Return):
+            self.telnet_send_command()
+        return
 
+    def new_server(self, widget=None):
+        """ Get Telnet server host and login details specified in the Gtk.Entry boxes in the Telnet connection dialog and attempt a connection. """
+
+        # Get connection details.
+        tcd = TelnetConnectionDialog(self.application)
+        response = tcd.dialog.run()
         if(response == Gtk.ResponseType.OK):
-            connection_info = dialog.get_connection_info()
-            host = connection_info["HOST"].get_text()
-            port = connection_info["PORT"].get_text()
-            username = connection_info["USERNAME"].get_text()
-            password = connection_info["PASSWORD"].get_text()
+            host = tcd.host
+            port = tcd.port
+            username = tcd.username
+            password = tcd.password
+            bookmark = tcd.bookmark
+            tcd.dialog.destroy()
+
+            # Handle empty hostname.
+            if(not host):
+                logging.error("No hostname specified.")
+                return
+
+            # Handle empty port number.
+            if(not port):
+                logging.warning("No port specified. Assuming default port 23...")
+                port = 23
+            else:
+                try:
+                    # Cast port into an int.
+                    port = int(port)
+                except ValueError as e:
+                    logging.error("Could not cast the DX cluster's port information to an integer.")
+                    logging.exception(e)
+                    return
 
             # Save the server details in a new bookmark, if desired.
-            if(connection_info["BOOKMARK"].get_active()):
+            if(bookmark):
                 try:
                     config = configparser.ConfigParser()
                     config.read(BOOKMARKS_FILE)
 
                     # Use the host name as the bookmark's identifier.
+                    if(username):
+                        bookmark_identifier = "%s@%s:%d" % (username, host, port)
+                    else:
+                        bookmark_identifier = "%s:%d" % (host, port)
+                    logging.debug("Using %s as the bookmark identifier." % bookmark_identifier)
+
+                    # Add bookmark.
                     try:
-                        config.add_section(host)
+                        config.add_section(bookmark_identifier)
                     except configparser.DuplicateSectionError:
                         # If the hostname already exists, assume the user wants to update the port number, username and/or password.
-                        logging.warning("A server with hostname '%s' already exists. Over-writing existing details..." % (host))
-                    config.set(host, "host", host)
-                    config.set(host, "port", port)
-                    config.set(host, "username", username)
-                    config.set(host, "password", password)
+                        logging.warning("Bookmark '%s' already exists. Over-writing existing details..." % (bookmark_identifier))
+                    config.set(bookmark_identifier, "host", host)
+                    config.set(bookmark_identifier, "port", str(port))
+                    config.set(bookmark_identifier, "username", username)
+                    config.set(bookmark_identifier, "password", password)
 
                     # Write the bookmarks to file.
                     if not os.path.exists(os.path.expanduser('~/.config/pyqso')):
@@ -154,49 +139,45 @@ class DXCluster(Gtk.VBox):
                     with open(BOOKMARKS_FILE, 'w') as f:
                         config.write(f)
 
-                    self._populate_bookmarks()
+                    self.populate_bookmarks()
 
                 except IOError:
                     # Maybe the bookmarks file could not be written to?
                     logging.error("Bookmark could not be saved. Check bookmarks file permissions? Going ahead with the server connection anyway...")
 
-            dialog.destroy()
-
-            try:
-                # Convert port (currently of type str) into an int.
-                port = int(port)
-                # Attempt a connection with the server.
-                self.telnet_connect(host, port, username, password)
-            except ValueError as e:
-                logging.error("Could not convert the server's port information to an integer.")
-                logging.exception(e)
+            # Attempt a connection with the server.
+            self.telnet_connect(host, port, username, password)
 
         else:
-            dialog.destroy()
+            tcd.dialog.destroy()
         return
 
-    def _populate_bookmarks(self):
+    def populate_bookmarks(self):
         """ Populate the list of bookmarked Telnet servers in the menu. """
+
+        # Get the bookmarks submenu.
+        subm_bookmarks = self.builder.get_object("subm_bookmarks")
+
         config = configparser.ConfigParser()
         have_config = (config.read(BOOKMARKS_FILE) != [])
 
         if(have_config):
             try:
                 # Clear the menu of all current bookmarks.
-                for i in self.subm_bookmarks.get_children():
-                    self.subm_bookmarks.remove(i)
+                for i in subm_bookmarks.get_children():
+                    subm_bookmarks.remove(i)
 
                 # Add all bookmarks in the config file.
                 for bookmark in config.sections():
                     mitem = Gtk.MenuItem(label=bookmark)
                     mitem.connect("activate", self.bookmarked_server, bookmark)
-                    self.subm_bookmarks.append(mitem)
+                    subm_bookmarks.append(mitem)
 
             except Exception as e:
                 logging.error("An error occurred whilst populating the DX cluster bookmarks menu.")
                 logging.exception(e)
 
-            self.show_all()  # Need to do this to update the bookmarks list in the menu.
+            self.builder.get_object("dx_cluster").show_all()  # Need to do this to update the bookmarks list in the menu.
 
         return
 
@@ -224,7 +205,7 @@ class DXCluster(Gtk.VBox):
         except IOError as e:
             logging.exception(e)
         except Exception as e:
-            logging.error("Could not connect to Telnet server '%s'" % name)
+            logging.error("Could not connect to Telnet server '%s'." % name)
             logging.exception(e)
 
         return
@@ -238,14 +219,18 @@ class DXCluster(Gtk.VBox):
         :arg str password: The user's password. This is an optional argument.
         """
 
-        if(host == "" or host is None):
-            logging.error("No Telnet server specified.")
+        # Handle empty host/port string (or the case where host/port are None).
+        if(not host):
+            error(parent=self.application.window, message="Unable to connect to a DX cluster because no hostname was specified.")
             return
-        if(port == "" or port is None):
-            port = 23  # Use the default Telnet port
+        if(not port):
+            logging.warning("No port specified. Assuming default port 23...")
+            port = 23  # Use the default Telnet port.
 
         try:
+            logging.debug("Attempting connection to Telnet server %s:%d..." % (host, port))
             self.connection = telnetlib.Telnet(host, port)
+            assert(self.connection)
 
             if(username):
                 self.connection.read_until("login: ".encode())
@@ -254,14 +239,17 @@ class DXCluster(Gtk.VBox):
                 self.connection.read_until("password: ".encode())
                 self.connection.write((password + "\n").encode())
         except Exception as e:
-            logging.error("Could not create a connection to the Telnet server")
+            message = "Could not create a connection to the Telnet server %s:%d. Check connection to the internets? Check connection details?" % (host, port)
+            error(parent=self.application.window, message=message)
             logging.exception(e)
             self.connection = None
             return
 
+        logging.debug("Connection to %s:%d established." % (host, port))
+
         self.set_items_sensitive(False)
 
-        self.check_io_event = GObject.timeout_add(1000, self._on_telnet_io)
+        self.check_io_event = GObject.timeout_add(1000, self.on_telnet_io)
 
         return
 
@@ -285,18 +273,20 @@ class DXCluster(Gtk.VBox):
     def telnet_send_command(self, widget=None):
         """ Send the user-specified command in the Gtk.Entry box to the Telnet server (if PyQSO is connected to one). """
         if(self.connection):
-            self.connection.write((self.command.get_text() + "\n").encode())
-            self.command.set_text("")
+            command = self.builder.get_object("command")
+            self.connection.write((command.get_text() + "\n").encode())
+            command.set_text("")
         return
 
-    def _on_telnet_io(self):
+    def on_telnet_io(self):
         """ Retrieve any new data from the Telnet server and print it out in the Gtk.TextView widget.
 
         :returns: Always returns True to satisfy the GObject timer.
         :rtype: bool
         """
         if(self.connection):
-            text = self.connection.read_very_eager().decode()
+            text = self.connection.read_very_eager()
+            text = text.decode("ascii", "replace")  # Replace any characters that cannot be decoded with a replacement marker.
             try:
                 text = text.replace("\u0007", "")  # Remove the BEL Unicode character from the end of the line
             except UnicodeDecodeError:
@@ -324,31 +314,5 @@ class DXCluster(Gtk.VBox):
         """
         self.items["CONNECT"].set_sensitive(sensitive)
         self.items["DISCONNECT"].set_sensitive(not sensitive)
-        self.send.set_sensitive(not sensitive)
+        self.items["SEND"].set_sensitive(not sensitive)
         return
-
-
-class TestDXCluster(unittest.TestCase):
-
-    """ The unit tests for the DXCluster class. """
-
-    def setUp(self):
-        """ Set up the objects needed for the unit tests. """
-        self.dxcluster = DXCluster(parent=None)
-
-    def tearDown(self):
-        """ Destroy any unit test resources. """
-        pass
-
-    def test_on_telnet_io(self):
-        """ Check that the response from the Telnet server can be correctly decoded. """
-
-        telnetlib.Telnet = unittest.mock.Mock(spec=telnetlib.Telnet)
-        connection = telnetlib.Telnet("hello", "world")
-        connection.read_very_eager.return_value = b"Test message from the Telnet server."
-        self.dxcluster.connection = connection
-        result = self.dxcluster._on_telnet_io()
-        assert(result)
-
-if(__name__ == '__main__'):
-    unittest.main()
