@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-#    Copyright (C) 2012-2017 Christian Thomas Jacobs.
+#    Copyright (C) 2012-2018 Christian Thomas Jacobs.
 
 #    This file is part of PyQSO.
 
@@ -20,6 +20,7 @@
 from gi.repository import Gtk
 import logging
 import sqlite3 as sqlite
+import json
 from os.path import expanduser
 try:
     import configparser
@@ -247,6 +248,14 @@ class Logbook:
             self.application.menu.set_record_items_sensitive(True)
         return
 
+    def on_button_release_event(self, treeview, event):
+        """ Show a popup menu when the user right-clicks a record in the logbook. """
+
+        if(event.button == 3):
+            self.application.popup.menu.popup(None, None, None, None, event.button, event.time)
+            self.application.popup.menu.show_all()
+            return True
+
     def new_log(self, widget=None):
         """ Create a new log in the logbook. """
 
@@ -382,8 +391,10 @@ class Logbook:
         self.treeview.append(Gtk.TreeView(model=self.sorter[index]))
         self.treeview[index].set_grid_lines(Gtk.TreeViewGridLines.BOTH)
         self.treeview[index].connect("row-activated", self.edit_record_callback)
+        self.treeview[index].connect("button-release-event", self.on_button_release_event)
         self.treeselection.append(self.treeview[index].get_selection())
         self.treeselection[index].set_mode(Gtk.SelectionMode.SINGLE)
+
         # Allow the Log to be scrolled up/down.
         sw = Gtk.ScrolledWindow()
         sw.set_shadow_type(Gtk.ShadowType.ETCHED_IN)
@@ -643,12 +654,14 @@ class Logbook:
 
     def export_log_adif(self, widget=None):
         """ Export the log (that is currently selected) to an ADIF file. """
-        page_index = self.notebook.get_current_page()  # Get the index of the selected tab in the logbook.
-        if(page_index == 0):  # If we are on the Summary page...
-            logging.debug("No log currently selected!")
+        # Get the index of the selected tab in the logbook.
+        try:
+            log_index = self.get_log_index()
+            if(log_index is None):
+                raise ValueError("The log index could not be determined. Perhaps the Summary page is selected?")
+        except ValueError as e:
+            error(parent=self.application.window, message=e)
             return
-
-        log_index = self.get_log_index()
         log = self.logs[log_index]
 
         dialog = Gtk.FileChooserDialog("Export Log as ADIF",
@@ -703,12 +716,14 @@ class Logbook:
 
     def export_log_cabrillo(self, widget=None):
         """ Export the log (that is currently selected) to a Cabrillo file. """
-        page_index = self.notebook.get_current_page()  # Get the index of the selected tab in the logbook.
-        if(page_index == 0):  # If we are on the Summary page...
-            logging.debug("No log currently selected!")
+        # Get the index of the selected tab in the logbook.
+        try:
+            log_index = self.get_log_index()
+            if(log_index is None):
+                raise ValueError("The log index could not be determined. Perhaps the Summary page is selected?")
+        except ValueError as e:
+            error(parent=self.application.window, message=e)
             return
-
-        log_index = self.get_log_index()
         log = self.logs[log_index]
 
         dialog = Gtk.FileChooserDialog("Export Log as Cabrillo",
@@ -775,11 +790,14 @@ class Logbook:
         """ Print all the records in the log (that is currently selected).
         Note that only a few important fields are printed because of the restricted width of the page. """
 
-        page_index = self.notebook.get_current_page()  # Get the index of the selected tab in the logbook.
-        if(page_index == 0):  # If we are on the Summary page...
-            logging.debug("No log currently selected!")
+        # Get the index of the selected tab in the logbook.
+        try:
+            log_index = self.get_log_index()
+            if(log_index is None):
+                raise ValueError("The log index could not be determined. Perhaps the Summary page is selected?")
+        except ValueError as e:
+            error(parent=self.application.window, message=e)
             return
-        log_index = self.get_log_index()
         log = self.logs[log_index]
 
         # Retrieve the records.
@@ -798,7 +816,7 @@ class Logbook:
 
     def add_record_callback(self, widget):
         """ A callback function used to add a particular record/QSO. """
-        # Get the log index.
+        # Get the index of the selected tab in the logbook.
         try:
             log_index = self.get_log_index()
             if(log_index is None):
@@ -1034,6 +1052,66 @@ class Logbook:
 
         return
 
+    def pinpoint_callback(self, widget=None, path=None):
+        """ A callback function used to pinpoint the callsign on the world map. """
+
+        try:
+            log_index = self.get_log_index()
+            row_index = self.get_record_index()
+            if(log_index is None or row_index is None):
+                raise ValueError("Could not determine the log and/or record index.")
+            r = self.logs[log_index].get_record_by_index(row_index)
+        except ValueError as e:
+            logging.error(e)
+            return
+
+        self.application.toolbox.world_map.pinpoint(r)
+
+        return
+
+    def copy_callback(self, widget=None, path=None):
+        """ A callback function used to copy selected logs. """
+
+        try:
+            log_index = self.get_log_index()
+            row_index = self.get_record_index()
+            if(log_index is None or row_index is None):
+                raise ValueError("Could not determine the log and/or record index.")
+            r = self.logs[log_index].get_record_by_index(row_index)
+        except ValueError as e:
+            logging.error(e)
+            return
+
+        d = {}
+        for key in r.keys():
+            d[key.upper()] = r[key]
+        j = json.dumps(d)
+
+        self.application.clipboard.set_text(j, len(j))
+
+        return
+
+    def clipboard_text_received(self, clipboard, text, log):
+        r = json.loads(text)
+        log.add_record(r)
+        return
+
+    def paste_callback(self, widget=None, path=None):
+        """ A callback function used to paste selected logs. """
+
+        try:
+            log_index = self.get_log_index()
+            if(log_index is None):
+                raise ValueError("Could not determine the log index.")
+            l = self.logs[log_index]
+        except ValueError as e:
+            logging.error(e)
+            return
+
+        self.application.clipboard.request_text(self.clipboard_text_received, l)
+
+        return
+
     @property
     def log_count(self):
         """ Return the total number of logs in the logbook.
@@ -1074,7 +1152,7 @@ class Logbook:
         """ Given the name of a log, return its index in the list of Log objects.
 
         :arg str name: The name of the log. If None, use the name of the currently-selected log.
-        :returns: The index of the named log in the list of Log objects. Returns None is the log cannot be found.
+        :returns: The index of the named log in the list of Log objects. Returns None if the log cannot be found.
         :rtype: int
         """
         if(name is None):
@@ -1095,6 +1173,37 @@ class Logbook:
                 break
         return log_index
 
+    def get_record_index(self):
+        """ Return the index of the currently selected record.
+
+        :returns: The index of the currently selected record in the currently selected log. Returns None if the record or log cannot be found.
+        :rtype: int
+        """
+
+        # Get the index of the selected log.
+        try:
+            log_index = self.get_log_index()
+            if(log_index is None):
+                raise ValueError("The log index could not be determined. Perhaps the Summary page is selected?")
+        except ValueError as e:
+            logging.error(e)
+            return None
+        log = self.logs[log_index]
+
+        # Get the selected row in the log.
+        (sort_model, path) = self.treeselection[log_index].get_selected_rows()
+        try:
+            sort_iter = sort_model.get_iter(path[0])
+            filter_iter = self.sorter[log_index].convert_iter_to_child_iter(sort_iter)
+            # ...and the ListStore model (i.e. the log) is a child of the filter model.
+            child_iter = self.filter[log_index].convert_iter_to_child_iter(filter_iter)
+            row_index = log.get_value(child_iter, 0)
+        except IndexError:
+            logging.error("Could not find the selected row's index!")
+            return None
+
+        return row_index
+
     def get_logs(self):
         """ Retrieve all the logs in the logbook file, and create Log objects that represent them.
 
